==> kaffe-0.8.0.tgz: kaffe-0.8.0/packages/tjwassoc.demon.co.uk/APIcore/lib/java.lang/System.c <==
/*
 * java.lang.System.c
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>, 1996.
 */

#define	DBG(s)

#define __jtypes_h
#include "config.h"
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include "config-std.h"
#include "config-io.h"
#if defined(HAVE_SYS_UTSNAME_H)
#include <sys/utsname.h>
#endif
#if defined(HAVE_PWD_H)
#include <pwd.h>
#endif
#include "../../../../kaffe/kaffevm/gtypes.h"
#include "../../../../kaffe/kaffevm/object.h"
#include <native.h>
#include "system.h"
#include "defs.h"
#include "java.lang.stubs/System.h"
#include "../../../../kaffe/kaffevm/constants.h"
#include "../../../../kaffe/kaffevm/access.h"
#include "../../../../kaffe/kaffevm/classMethod.h"
#include "../../../../kaffe/kaffevm/support.h"
#include "../../../../kaffe/kaffevm/soft.h"

static char cwdpath[MAXPATHLEN];

extern jlong currentTime(void);


IMPORT(userProperty*) userProperties;

/*
 * Copy one part of an array to another.
 */
void
java_lang_System_arraycopy(struct Hjava_lang_System* none, struct Hjava_lang_Object* src, jint srcpos, struct Hjava_lang_Object* dst, jint dstpos, jint len)
{
	char* in;
	char* out;
	int elemsz;
	Class* sclass;
	Class* dclass;
	char* sig;

	sclass = OBJECT_CLASS(src);
	dclass = OBJECT_CLASS(dst);

	/* Must be some array type */
	assert(CLASS_IS_ARRAY(dclass));
	elemsz = TYPE_SIZE(CLASS_ELEMENT_TYPE(dclass));
	sig = CLASS_CNAME(dclass);

	len *= elemsz;
	srcpos *= elemsz;
	dstpos *= elemsz;

	in = &((char*)(src+1))[srcpos];
	out = &((char*)(dst+1))[dstpos];


	/* If I can't cast the src array to the dst array then
	 * things could be difficult.  I can try to copy the elements
	 * individually so long as each dst element can be cast the the src.
	 */
	if (instanceof_array(dclass, sclass) == 0) {
DBG(		printf("CASTING %s -> %s\n", CLASS_CNAME(sclass), sig);	)
		sig++;
		dclass = classFromSig(&sig, 0);
		if (dclass == 0) {
			SignalError(0, "java.lang.ClassNotFoundException", CLASS_CNAME(dclass));
		}
		for (; len > 0; len -= sizeof(Object**)) {
			soft_checkcast(dclass, *(Object**)in);
			*(Object**)out = *(Object**)in;
			in += sizeof(Object*);
			out += sizeof(Object*);
		}
	}

#if defined(HAVE_MEMMOVE)
	memmove((void*)out, (void*)in, len);
#else
	/* Do it ourself */
	if (out < in) {
		/* Copy forwards */
		for (; len > 0; len--) {
			*out++ = *in++;
		}
	}
	else {
		/* Copy backwards */
		out += len;
		in += len;
		for (; len > 0; len--) {
			*--out = *--in;
		}
	}
#endif
}

/*
 * Initialise system properties to their defaults.
 */
struct Hjava_util_Properties*
java_lang_System_initProperties(struct Hjava_lang_System* none, struct Hjava_util_Properties* p)
{
	int r;
	char* jhome;
	char* cpath;
	char* dir;
	char* jresources;
	userProperty* prop;
#if defined(HAVE_SYS_UTSNAME_H)
	struct utsname system;
#endif
#if defined(HAVE_PWD_H)
	struct passwd* pw;
#endif

	/* Add the default properties:
	 * java.version		Java version number
	 * java.vendor          Java vendor specific string
	 * java.vendor.url      Java vendor URL
	 * java.home            Java installation directory
	 * java.class.version   Java class version number
	 * java.class.path      Java classpath
	 * os.name              Operating System Name
	 * os.arch              Operating System Architecture
	 * os.version           Operating System Version
	 * file.separator       File separator ("/" on Unix)
	 * path.separator       Path separator (":" on Unix)
	 * line.separator       Line separator ("\n" on Unix)
	 * user.name            User account name
	 * user.home            User home directory
	 * user.dir             User's current working directory
	 */

	setProperty(p, "java.version", kaffe_version);
	setProperty(p, "java.vendor", kaffe_vendor);
	setProperty(p, "java.vendor.url", kaffe_vendor_url);

	jhome = getenv(KAFFEHOME);
	if (jhome == 0) {
		jhome = ".";
	}
	setProperty(p, "java.home", jhome);

	setProperty(p, "java.class.version", kaffe_class_version);

	cpath = getenv(KAFFECLASSPATH);
	if (cpath == 0) {
#if defined(DEFAULT_CLASSPATH)
		cpath = DEFAULT_CLASSPATH;
#else
		cpath = ".";
#endif
	}
	setProperty(p, "java.class.path", cpath);

	setProperty(p, "file.separator", file_seperator);
	setProperty(p, "path.separator", path_seperator);
	setProperty(p, "line.separator", line_seperator);

#if defined(HAVE_GETCWD)
	dir = getcwd(cwdpath, MAXPATHLEN);
#elif defined(HAVE_GETWD)
	dir = getwd(cwdpath);
#else
	dir = 0;	/* Cannot get current directory */
#endif
	if (dir == 0) {
		dir = ".";
	}
	setProperty(p, "user.dir", dir);

#if defined(HAVE_SYS_UTSNAME_H) && defined(HAVE_UNAME)
	/* Setup system properties */
	r = uname(&system);
	assert(r >= 0);
	setProperty(p, "os.name", system.sysname);
	setProperty(p, "os.arch", system.machine);
	setProperty(p, "os.version", system.version);
#else
	setProperty(p, "os.name", "Unknown");
	setProperty(p, "os.arch", "Unknown");
	setProperty(p, "os.version", "Unknown");
#endif
#if defined(HAVE_PWD_H) && defined(HAVE_GETUID)
	/* Setup user properties */
	pw = getpwuid(getuid());
	if (pw != 0) {
		setProperty(p, "user.name", pw->pw_name);
		setProperty(p, "user.home", pw->pw_dir);
	}
	else
#endif
	{
		setProperty(p, "user.name", "Unknown");
		setProperty(p, "user.home", "Unknown");
	}

	/* Add in the awt.Toolkit we will be using */
#if defined(HAVE_AWT_TOOLKIT_BISS)
	setProperty(p, "awt.toolkit", "biss.awt.kernel.Toolkit");
	jresources = checked_malloc(strlen(jhome) + strlen("/lib") + 1);
	strcpy(jresources, jhome);
	strcat(jresources, "/lib");
	setProperty(p, "java.resources", jresources);
	checked_free(jresources);
#elif defined(HAVE_AWT_TOOLKIT_SAWT)
	setProperty(p, "awt.toolkit", "kaffe.awt.simple.SimpleToolkit");
#endif

	/* Now process user defined properties */
	for (prop = userProperties; prop != 0; prop = prop->next) {
		setProperty(p, prop->key, prop->value);
	}

	return (p);
}

/*
 * Return current time.
 */
jlong
java_lang_System_currentTimeMillis(struct Hjava_lang_System* none)
{
	return (currentTime());
}

==> kaffe-0.8.1.tgz: kaffe-0.8.1/packages/tjwassoc.demon.co.uk/APIcore/lib/java.lang/System.c <==
/*
 * java.lang.System.c
 *
 * Copyright (c) 1996 T. J. Wilkinson & Associates, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@tjwassoc.demon.co.uk>, 1996.
 */

#define	DBG(s)

#include "config.h"
#include "config-std.h"
#include "config-io.h"
#include "config-mem.h"
#if defined(HAVE_SYS_UTSNAME_H)
#include <sys/utsname.h>
#endif
#if defined(HAVE_PWD_H)
#include <pwd.h>
#endif
#include "../../../../kaffe/kaffevm/gtypes.h"
#include "../../../../kaffe/kaffevm/object.h"
#include <native.h>
#include "system.h"
#include "defs.h"
#include "java.lang.stubs/System.h"
#include "../../../../kaffe/kaffevm/constants.h"
#include "../../../../kaffe/kaffevm/access.h"
#include "../../../../kaffe/kaffevm/classMethod.h"
#include "../../../../kaffe/kaffevm/support.h"
#include "../../../../kaffe/kaffevm/soft.h"

static char cwdpath[MAXPATHLEN];

extern jlong currentTime(void);


IMPORT(userProperty*) userProperties;

/*
 * Copy one part of an array to another.
 */
void
java_lang_System_arraycopy(struct Hjava_lang_System* none, struct Hjava_lang_Object* src, jint srcpos, struct Hjava_lang_Object* dst, jint dstpos, jint len)
{
	char* in;
	char* out;
	int elemsz;
	Class* sclass;
	Class* dclass;
	char* sig;

	sclass = OBJECT_CLASS(src);
	dclass = OBJECT_CLASS(dst);

	/* Must be arrays */
	if (CLASS_IS_ARRAY(sclass) == 0 || CLASS_IS_ARRAY(dclass) == 0) {
		/* What error should I really generate here? */
		SignalError(0, "java.lang.ArrayIndexOutOfBoundsException", "");
	}

	/* Make sure we'll keep in the array boundaries */
	if ((srcpos < 0 || srcpos + len > ARRAY_SIZE(src)) ||
	    (dstpos < 0 || dstpos + len > ARRAY_SIZE(dst)) ||
	    (len < 0)) {
		SignalError(0, "java.lang.ArrayIndexOutOfBoundsException", "");
	}

	elemsz = TYPE_SIZE(CLASS_ELEMENT_TYPE(dclass));
	sig = CLASS_CNAME(dclass);

	len *= elemsz;
	srcpos *= elemsz;
	dstpos *= elemsz;

	in = &((char*)(src+1))[srcpos];
	out = &((char*)(dst+1))[dstpos];


	/* If I can't cast the src array to the dst array then
	 * things could be difficult.  I can try to copy the elements
	 * individually so long as each dst element can be cast the the src.
	 */
	if (instanceof_array(dclass, sclass) == 0) {
DBG(		printf("CASTING %s -> %s\n", CLASS_CNAME(sclass), sig);	)
		sig++;
		dclass = classFromSig(&sig, 0);
		if (dclass == 0) {
			SignalError(0, "java.lang.ClassNotFoundException", CLASS_CNAME(dclass));
		}
		for (; len > 0; len -= sizeof(Object**)) {
			soft_checkcast(dclass, *(Object**)in);
			*(Object**)out = *(Object**)in;
			in += sizeof(Object*);
			out += sizeof(Object*);
		}
	}

#if defined(HAVE_MEMMOVE)
	memmove((void*)out, (void*)in, len);
#else
	/* Do it ourself */
	if (out < in) {
		/* Copy forwards */
		for (; len > 0; len--) {
			*out++ = *in++;
		}
	}
	else {
		/* Copy backwards */
		out += len;
		in += len;
		for (; len > 0; len--) {
			*--out = *--in;
		}
	}
#endif
}

/*
 * Initialise system properties to their defaults.
 */
struct Hjava_util_Properties*
java_lang_System_initProperties(struct Hjava_lang_System* none, struct Hjava_util_Properties* p)
{
	int r;
	char* jhome;
	char* cpath;
	char* dir;
	char* jresources;
	userProperty* prop;
#if defined(HAVE_SYS_UTSNAME_H)
	struct utsname system;
#endif
#if defined(HAVE_PWD_H)
	struct passwd* pw;
#endif

	/* Add the default properties:
	 * java.version		Java version number
	 * java.vendor          Java vendor specific string
	 * java.vendor.url      Java vendor URL
	 * java.home            Java installation directory
	 * java.class.version   Java class version number
	 * java.class.path      Java classpath
	 * os.name              Operating System Name
	 * os.arch              Operating System Architecture
	 * os.version           Operating System Version
	 * file.separator       File separator ("/" on Unix)
	 * path.separator       Path separator (":" on Unix)
	 * line.separator       Line separator ("\n" on Unix)
	 * user.name            User account name
	 * user.home            User home directory
	 * user.dir             User's current working directory
	 */

	setProperty(p, "java.version", kaffe_version);
	setProperty(p, "java.vendor", kaffe_vendor);
	setProperty(p, "java.vendor.url", kaffe_vendor_url);

	jhome = getenv(KAFFEHOME);
	if (jhome == 0) {
		jhome = ".";
	}
	setProperty(p, "java.home", jhome);

	setProperty(p, "java.class.version", kaffe_class_version);

	cpath = getenv(KAFFECLASSPATH);
	if (cpath == 0) {
#if defined(DEFAULT_CLASSPATH)
		cpath = DEFAULT_CLASSPATH;
#else
		cpath = ".";
#endif
	}
	setProperty(p, "java.class.path", cpath);

	setProperty(p, "file.separator", file_seperator);
	setProperty(p, "path.separator", path_seperator);
	setProperty(p, "line.separator", line_seperator);

#if defined(HAVE_GETCWD)
	dir = getcwd(cwdpath, MAXPATHLEN);
#elif defined(HAVE_GETWD)
	dir = getwd(cwdpath);
#else
	dir = 0;	/* Cannot get current directory */
#endif
	if (dir == 0) {
		dir = ".";
	}
	setProperty(p, "user.dir", dir);

#if defined(HAVE_SYS_UTSNAME_H) && defined(HAVE_UNAME)
	/* Setup system properties */
	r = uname(&system);
	assert(r >= 0);
	setProperty(p, "os.name", system.sysname);
	setProperty(p, "os.arch", system.machine);
	setProperty(p, "os.version", system.version);
#else
	setProperty(p, "os.name", "Unknown");
	setProperty(p, "os.arch", "Unknown");
	setProperty(p, "os.version", "Unknown");
#endif
#if defined(HAVE_PWD_H) && defined(HAVE_GETUID)
	/* Setup user properties */
	pw = getpwuid(getuid());
	if (pw != 0) {
		setProperty(p, "user.name", pw->pw_name);
		setProperty(p, "user.home", pw->pw_dir);
	}
	else
#endif
	{
		setProperty(p, "user.name", "Unknown");
		setProperty(p, "user.home", "Unknown");
	}

	/* Add in the awt.Toolkit we will be using */
#if defined(HAVE_PACKAGE_BISS_NET_COM)
	setProperty(p, "awt.toolkit", "biss.awt.kernel.Toolkit");
#elif defined(HAVE_PACKAGE_EPFL_CH)
	setProperty(p, "awt.toolkit", "kaffe.awt.simple.SimpleToolkit");
#endif
#if defined(HAVE_AWT_TOOLKIT_BISS) || defined(HAVE_AWT_TOOLKIT_SAWT)
	jresources = checked_malloc(strlen(jhome) + strlen("/lib") + 1);
	strcpy(jresources, jhome);
	strcat(jresources, "/lib");
	setProperty(p, "java.resources", jresources);
	checked_free(jresources);
#endif

	/* Now process user defined properties */
	for (prop = userProperties; prop != 0; prop = prop->next) {
		setProperty(p, prop->key, prop->value);
	}

	return (p);
}

/*
 * Return current time.
 */
jlong
java_lang_System_currentTimeMillis(struct Hjava_lang_System* none)
{
	return (currentTime());
}
